@@ -4,7 +4,6 @@
 #include "date.h"
 #include "gettext.h"
 #include "hex.h"
-#include "refs.h"
 #include "object-store.h"
 #include "pkt-line.h"
 #include "sideband.h"
@@ -12,10 +11,8 @@
 #include "remote.h"
 #include "connect.h"
 #include "send-pack.h"
-#include "quote.h"
 #include "transport.h"
 #include "version.h"
-#include "wrapper.h"
 #include "oid-array.h"
 #include "gpg-interface.h"
 #include "shallow.h"
@@ -45,22 +42,10 @@ int option_parse_push_signed(const struct option *opt,
 	die("bad %s argument: %s", opt->long_name, arg);
 }
 
-static int config_use_sideband = 1;
-
-static int send_pack_config(const char *var, const char *value, void *unused)
+static void feed_object(struct repository *r,
+			const struct object_id *oid, FILE *fh, int negative)
 {
-	if (!strcmp("sendpack.sideband", var))
-		config_use_sideband = git_config_bool(var, value);
-
-	return 0;
-}
-
-static void feed_object(const struct object_id *oid, FILE *fh, int negative)
-{
-	if (negative &&
-	    !repo_has_object_file_with_flags(the_repository, oid,
-					     OBJECT_INFO_SKIP_FETCH_OBJECT |
-					     OBJECT_INFO_QUICK))
+	if (negative && !has_object(r, oid, 0))
 		return;
 
 	if (negative)
@@ -72,7 +57,8 @@ static void feed_object(const struct object_id *oid, FILE *fh, int negative)
 /*
  * Make a pack stream and spit it out into file descriptor fd
  */
-static int pack_objects(int fd, struct ref *refs, struct oid_array *advertised,
+static int pack_objects(struct repository *r,
+			int fd, struct ref *refs, struct oid_array *advertised,
 			struct oid_array *negotiated,
 			struct send_pack_args *args)
 {
@@ -83,9 +69,9 @@ static int pack_objects(int fd, struct ref *refs, struct oid_array *advertised,
 	 */
 	struct child_process po = CHILD_PROCESS_INIT;
 	FILE *po_in;
-	int i;
 	int rc;
 
+	trace2_region_enter("send_pack", "pack_objects", r);
 	strvec_push(&po.args, "pack-objects");
 	strvec_push(&po.args, "--all-progress-implied");
 	strvec_push(&po.args, "--revs");
@@ -98,7 +84,7 @@ static int pack_objects(int fd, struct ref *refs, struct oid_array *advertised,
 		strvec_push(&po.args, "-q");
 	if (args->progress)
 		strvec_push(&po.args, "--progress");
-	if (is_repository_shallow(the_repository))
+	if (is_repository_shallow(r))
 		strvec_push(&po.args, "--shallow");
 	if (args->disable_bitmaps)
 		strvec_push(&po.args, "--no-use-bitmap-index");
@@ -114,16 +100,16 @@ static int pack_objects(int fd, struct ref *refs, struct oid_array *advertised,
 	 * parameters by writing to the pipe.
 	 */
 	po_in = xfdopen(po.in, "w");
-	for (i = 0; i < advertised->nr; i++)
-		feed_object(&advertised->oid[i], po_in, 1);
-	for (i = 0; i < negotiated->nr; i++)
-		feed_object(&negotiated->oid[i], po_in, 1);
+	for (size_t i = 0; i < advertised->nr; i++)
+		feed_object(r, &advertised->oid[i], po_in, 1);
+	for (size_t i = 0; i < negotiated->nr; i++)
+		feed_object(r, &negotiated->oid[i], po_in, 1);
 
 	while (refs) {
 		if (!is_null_oid(&refs->old_oid))
-			feed_object(&refs->old_oid, po_in, 1);
+			feed_object(r, &refs->old_oid, po_in, 1);
 		if (!is_null_oid(&refs->new_oid))
-			feed_object(&refs->new_oid, po_in, 0);
+			feed_object(r, &refs->new_oid, po_in, 0);
 		refs = refs->next;
 	}
 
@@ -157,8 +143,10 @@ static int pack_objects(int fd, struct ref *refs, struct oid_array *advertised,
 		 */
 		if (rc > 128 && rc != 141)
 			error("pack-objects died of signal %d", rc - 128);
+		trace2_region_leave("send_pack", "pack_objects", r);
 		return -1;
 	}
+	trace2_region_leave("send_pack", "pack_objects", r);
 	return 0;
 }
 
@@ -173,7 +161,8 @@ static int receive_unpack_status(struct packet_reader *reader)
 	return 0;
 }
 
-static int receive_status(struct packet_reader *reader, struct ref *refs)
+static int receive_status(struct repository *r,
+			  struct packet_reader *reader, struct ref *refs)
 {
 	struct ref *hint;
 	int ret;
@@ -181,6 +170,7 @@ static int receive_status(struct packet_reader *reader, struct ref *refs)
 	int new_report = 0;
 	int once = 0;
 
+	trace2_region_enter("send_pack", "receive_status", r);
 	hint = NULL;
 	ret = receive_unpack_status(reader);
 	while (1) {
@@ -229,10 +219,10 @@ static int receive_status(struct packet_reader *reader, struct ref *refs)
 			if (!strcmp(key, "refname"))
 				report->ref_name = xstrdup_or_null(val);
 			else if (!strcmp(key, "old-oid") && val &&
-				 !parse_oid_hex(val, &old_oid, &val))
+				 !parse_oid_hex_algop(val, &old_oid, &val, r->hash_algo))
 				report->old_oid = oiddup(&old_oid);
 			else if (!strcmp(key, "new-oid") && val &&
-				 !parse_oid_hex(val, &new_oid, &val))
+				 !parse_oid_hex_algop(val, &new_oid, &val, r->hash_algo))
 				report->new_oid = oiddup(&new_oid);
 			else if (!strcmp(key, "forced-update"))
 				report->forced_update = 1;
@@ -272,13 +262,14 @@ static int receive_status(struct packet_reader *reader, struct ref *refs)
 			if (p)
 				hint->remote_status = xstrdup(p);
 			else
-				hint->remote_status = "failed";
+				hint->remote_status = xstrdup("failed");
 		} else {
 			hint->status = REF_STATUS_OK;
 			hint->remote_status = xstrdup_or_null(p);
 			new_report = 1;
 		}
 	}
+	trace2_region_leave("send_pack", "receive_status", r);
 	return ret;
 }
 
@@ -300,9 +291,9 @@ static int advertise_shallow_grafts_cb(const struct commit_graft *graft, void *c
 	return 0;
 }
 
-static void advertise_shallow_grafts_buf(struct strbuf *sb)
+static void advertise_shallow_grafts_buf(struct repository *r, struct strbuf *sb)
 {
-	if (!is_repository_shallow(the_repository))
+	if (!is_repository_shallow(r))
 		return;
 	for_each_commit_graft(advertise_shallow_grafts_cb, sb);
 }
@@ -359,7 +350,8 @@ static int generate_push_cert(struct strbuf *req_buf,
 {
 	const struct ref *ref;
 	struct string_list_item *item;
-	char *signing_key_id = xstrdup(get_signing_key_id());
+	char *signing_key_id = get_signing_key_id();
+	char *signing_key = get_signing_key();
 	const char *cp, *np;
 	struct strbuf cert = STRBUF_INIT;
 	int update_seen = 0;
@@ -392,7 +384,7 @@ static int generate_push_cert(struct strbuf *req_buf,
 	if (!update_seen)
 		goto free_return;
 
-	if (sign_buffer(&cert, &cert, get_signing_key()))
+	if (sign_buffer(&cert, &cert, signing_key))
 		die(_("failed to sign the push certificate"));
 
 	packet_buf_write(req_buf, "push-cert%c%s", 0, cap_string);
@@ -405,6 +397,7 @@ static int generate_push_cert(struct strbuf *req_buf,
 
 free_return:
 	free(signing_key_id);
+	free(signing_key);
 	strbuf_release(&cert);
 	return update_seen;
 }
@@ -431,24 +424,34 @@ static void reject_invalid_nonce(const char *nonce, int len)
 	}
 }
 
-static void get_commons_through_negotiation(const char *url,
+static void get_commons_through_negotiation(struct repository *r,
+					    const char *url,
 					    const struct ref *remote_refs,
 					    struct oid_array *commons)
 {
 	struct child_process child = CHILD_PROCESS_INIT;
 	const struct ref *ref;
-	int len = the_hash_algo->hexsz + 1; /* hash + NL */
+	int len = r->hash_algo->hexsz + 1; /* hash + NL */
+	int nr_negotiation_tip = 0;
 
 	child.git_cmd = 1;
 	child.no_stdin = 1;
 	child.out = -1;
 	strvec_pushl(&child.args, "fetch", "--negotiate-only", NULL);
 	for (ref = remote_refs; ref; ref = ref->next) {
-		if (!is_null_oid(&ref->new_oid))
-			strvec_pushf(&child.args, "--negotiation-tip=%s", oid_to_hex(&ref->new_oid));
+		if (!is_null_oid(&ref->new_oid)) {
+			strvec_pushf(&child.args, "--negotiation-tip=%s",
+				     oid_to_hex(&ref->new_oid));
+			nr_negotiation_tip++;
+		}
 	}
 	strvec_push(&child.args, url);
 
+	if (!nr_negotiation_tip) {
+		child_process_clear(&child);
+		return;
+	}
+
 	if (start_command(&child))
 		die(_("send-pack: unable to fork off fetch subprocess"));
 
@@ -462,7 +465,7 @@ static void get_commons_through_negotiation(const char *url,
 			break;
 		if (read_len != len)
 			die("invalid length read %d", read_len);
-		if (parse_oid_hex(hex_hash, &oid, &end) || *end != '\n')
+		if (parse_oid_hex_algop(hex_hash, &oid, &end, r->hash_algo) || *end != '\n')
 			die("invalid hash");
 		oid_array_append(commons, &oid);
 	} while (1);
@@ -476,7 +479,8 @@ static void get_commons_through_negotiation(const char *url,
 	}
 }
 
-int send_pack(struct send_pack_args *args,
+int send_pack(struct repository *r,
+	      struct send_pack_args *args,
 	      int fd[], struct child_process *conn,
 	      struct ref *remote_refs,
 	      struct oid_array *extra_have)
@@ -490,7 +494,7 @@ int send_pack(struct send_pack_args *args,
 	int need_pack_data = 0;
 	int allow_deleting_refs = 0;
 	int status_report = 0;
-	int use_sideband = 0;
+	int use_sideband = 1;
 	int quiet_supported = 0;
 	int agent_supported = 0;
 	int advertise_sid = 0;
@@ -503,26 +507,29 @@ int send_pack(struct send_pack_args *args,
 	unsigned cmds_sent = 0;
 	int ret;
 	struct async demux;
-	const char *push_cert_nonce = NULL;
+	char *push_cert_nonce = NULL;
 	struct packet_reader reader;
 	int use_bitmaps;
 
 	if (!remote_refs) {
 		fprintf(stderr, "No refs in common and none specified; doing nothing.\n"
 			"Perhaps you should specify a branch.\n");
-		return 0;
+		ret = 0;
+		goto out;
 	}
 
-	git_config(send_pack_config, NULL);
+	repo_config_get_bool(r, "sendpack.sideband", &use_sideband);
+	repo_config_get_bool(r, "push.negotiate", &push_negotiate);
+	if (push_negotiate) {
+		trace2_region_enter("send_pack", "push_negotiate", r);
+		get_commons_through_negotiation(r, args->url, remote_refs, &commons);
+		trace2_region_leave("send_pack", "push_negotiate", r);
+	}
 
-	git_config_get_bool("push.negotiate", &push_negotiate);
-	if (push_negotiate)
-		get_commons_through_negotiation(args->url, remote_refs, &commons);
-
-	if (!git_config_get_bool("push.usebitmaps", &use_bitmaps))
+	if (!repo_config_get_bool(r, "push.usebitmaps", &use_bitmaps))
 		args->disable_bitmaps = !use_bitmaps;
 
-	git_config_get_bool("transfer.advertisesid", &advertise_sid);
+	repo_config_get_bool(r, "transfer.advertisesid", &advertise_sid);
 
 	/* Does the other end support the reporting? */
 	if (server_supports("report-status-v2"))
@@ -533,8 +540,7 @@ int send_pack(struct send_pack_args *args,
 		allow_deleting_refs = 1;
 	if (server_supports("ofs-delta"))
 		args->use_ofs_delta = 1;
-	if (config_use_sideband && server_supports("side-band-64k"))
-		use_sideband = 1;
+	use_sideband = use_sideband && server_supports("side-band-64k");
 	if (server_supports("quiet"))
 		quiet_supported = 1;
 	if (server_supports("agent"))
@@ -548,15 +554,16 @@ int send_pack(struct send_pack_args *args,
 	if (server_supports("push-options"))
 		push_options_supported = 1;
 
-	if (!server_supports_hash(the_hash_algo->name, &object_format_supported))
+	if (!server_supports_hash(r->hash_algo->name, &object_format_supported))
 		die(_("the receiving end does not support this repository's hash algorithm"));
 
 	if (args->push_cert != SEND_PACK_PUSH_CERT_NEVER) {
 		size_t len;
-		push_cert_nonce = server_feature_value("push-cert", &len);
-		if (push_cert_nonce) {
-			reject_invalid_nonce(push_cert_nonce, len);
-			push_cert_nonce = xmemdupz(push_cert_nonce, len);
+		const char *nonce = server_feature_value("push-cert", &len);
+
+		if (nonce) {
+			reject_invalid_nonce(nonce, len);
+			push_cert_nonce = xmemdupz(nonce, len);
 		} else if (args->push_cert == SEND_PACK_PUSH_CERT_ALWAYS) {
 			die(_("the receiving end does not support --signed push"));
 		} else if (args->push_cert == SEND_PACK_PUSH_CERT_IF_ASKED) {
@@ -589,7 +596,7 @@ int send_pack(struct send_pack_args *args,
 	if (use_push_options)
 		strbuf_addstr(&cap_buf, " push-options");
 	if (object_format_supported)
-		strbuf_addf(&cap_buf, " object-format=%s", the_hash_algo->name);
+		strbuf_addf(&cap_buf, " object-format=%s", r->hash_algo->name);
 	if (agent_supported)
 		strbuf_addf(&cap_buf, " agent=%s", git_user_agent_sanitized());
 	if (advertise_sid)
@@ -619,12 +626,12 @@ int send_pack(struct send_pack_args *args,
 			 * atomically, abort the whole operation.
 			 */
 			if (use_atomic) {
-				strbuf_release(&req_buf);
-				strbuf_release(&cap_buf);
 				reject_atomic_push(remote_refs, args->send_mirror);
-				error("atomic push failed for ref %s. status: %d\n",
+				error("atomic push failed for ref %s. status: %d",
 				      ref->name, ref->status);
-				return args->porcelain ? 0 : -1;
+				ret = ERROR_SEND_PACK_BAD_REF_STATUS;
+				packet_flush(out);
+				goto out;
 			}
 			/* else fallthrough */
 		default:
@@ -640,15 +647,16 @@ int send_pack(struct send_pack_args *args,
 	}
 
 	if (!args->dry_run)
-		advertise_shallow_grafts_buf(&req_buf);
+		advertise_shallow_grafts_buf(r, &req_buf);
 
 	/*
 	 * Finally, tell the other end!
 	 */
-	if (!args->dry_run && push_cert_nonce)
+	if (!args->dry_run && push_cert_nonce) {
 		cmds_sent = generate_push_cert(&req_buf, remote_refs, args,
 					       cap_buf.buf, push_cert_nonce);
-	else if (!args->dry_run)
+		trace2_printf("Generated push certificate");
+	} else if (!args->dry_run) {
 		for (ref = remote_refs; ref; ref = ref->next) {
 			char *old_hex, *new_hex;
 
@@ -668,6 +676,7 @@ int send_pack(struct send_pack_args *args,
 						 old_hex, new_hex, ref->name);
 			}
 		}
+	}
 
 	if (use_push_options) {
 		struct string_list_item *item;
@@ -678,7 +687,7 @@ int send_pack(struct send_pack_args *args,
 	}
 
 	if (args->stateless_rpc) {
-		if (!args->dry_run && (cmds_sent || is_repository_shallow(the_repository))) {
+		if (!args->dry_run && (cmds_sent || is_repository_shallow(r))) {
 			packet_buf_flush(&req_buf);
 			send_sideband(out, -1, req_buf.buf, req_buf.len, LARGE_PACKET_MAX);
 		}
@@ -686,8 +695,6 @@ int send_pack(struct send_pack_args *args,
 		write_or_die(out, req_buf.buf, req_buf.len);
 		packet_flush(out);
 	}
-	strbuf_release(&req_buf);
-	strbuf_release(&cap_buf);
 
 	if (use_sideband && cmds_sent) {
 		memset(&demux, 0, sizeof(demux));
@@ -705,7 +712,7 @@ int send_pack(struct send_pack_args *args,
 			   PACKET_READ_DIE_ON_ERR_PACKET);
 
 	if (need_pack_data && cmds_sent) {
-		if (pack_objects(out, remote_refs, extra_have, &commons, args) < 0) {
+		if (pack_objects(r, out, remote_refs, extra_have, &commons, args) < 0) {
 			if (args->stateless_rpc)
 				close(out);
 			if (git_connection_is_socket(conn))
@@ -718,14 +725,16 @@ int send_pack(struct send_pack_args *args,
 			 * we get one).
 			 */
 			if (status_report)
-				receive_status(&reader, remote_refs);
+				receive_status(r, &reader, remote_refs);
 
 			if (use_sideband) {
 				close(demux.out);
 				finish_async(&demux);
 			}
 			fd[1] = -1;
-			return -1;
+
+			ret = -1;
+			goto out;
 		}
 		if (!args->stateless_rpc)
 			/* Closed by pack_objects() via start_command() */
@@ -735,7 +744,7 @@ int send_pack(struct send_pack_args *args,
 		packet_flush(out);
 
 	if (status_report && cmds_sent)
-		ret = receive_status(&reader, remote_refs);
+		ret = receive_status(r, &reader, remote_refs);
 	else
 		ret = 0;
 	if (args->stateless_rpc)
@@ -750,10 +759,7 @@ int send_pack(struct send_pack_args *args,
 	}
 
 	if (ret < 0)
-		return ret;
-
-	if (args->porcelain)
-		return 0;
+		goto out;
 
 	for (ref = remote_refs; ref; ref = ref->next) {
 		switch (ref->status) {
@@ -762,8 +768,17 @@ int send_pack(struct send_pack_args *args,
 		case REF_STATUS_OK:
 			break;
 		default:
-			return -1;
+			ret = ERROR_SEND_PACK_BAD_REF_STATUS;
+			goto out;
 		}
 	}
-	return 0;
+
+	ret = 0;
+
+out:
+	oid_array_clear(&commons);
+	strbuf_release(&req_buf);
+	strbuf_release(&cap_buf);
+	free(push_cert_nonce);
+	return ret;
 }
